@@ -1,14 +1,22 @@
-from .utils import *
-from .vocabs import *
-from .funsd import *
+from doctr.file_utils import is_tf_available
+
+from .generator import *
 from .cord import *
 from .detection import *
-from .recognition import *
+from .doc_artefacts import *
+from .funsd import *
+from .ic03 import *
+from .ic13 import *
+from .iiit5k import *
+from .imgur5k import *
 from .ocr import *
+from .recognition import *
 from .sroie import *
-from .classification import *
-
-from doctr.file_utils import is_tf_available
+from .svhn import *
+from .svt import *
+from .synthtext import *
+from .utils import *
+from .vocabs import *
 
 if is_tf_available():
     from .loader import *
