@@ -1,4 +1,4 @@
-# Copyright (C) 2021, Mindee.
+# Copyright (C) 2021-2022, Mindee.
 
 # This program is licensed under the Apache License version 2.
 # See LICENSE or go to <https://www.apache.org/licenses/LICENSE-2.0.txt> for full license details.
@@ -20,10 +20,12 @@ class CORD(VisionDataset):
     """CORD dataset from `"CORD: A Consolidated Receipt Dataset forPost-OCR Parsing"
     <https://openreview.net/pdf?id=SJl3z659UH>`_.
 
-    Example::
-        >>> from doctr.datasets import CORD
-        >>> train_set = CORD(train=True, download=True)
-        >>> img, target = train_set[0]
+    .. image:: https://github.com/mindee/doctr/releases/download/v0.5.0/cord-grid.png
+        :align: center
+
+    >>> from doctr.datasets import CORD
+    >>> train_set = CORD(train=True, download=True)
+    >>> img, target = train_set[0]
 
     Args:
         train: whether the subset should be the training one
@@ -50,6 +52,7 @@ class CORD(VisionDataset):
         tmp_root = os.path.join(self.root, 'image')
         self.data: List[Tuple[str, Dict[str, Any]]] = []
         self.train = train
+        np_dtype = np.float32
         for img_path in os.listdir(tmp_root):
             # File existence check
             if not os.path.exists(os.path.join(tmp_root, img_path)):
@@ -65,14 +68,15 @@ class CORD(VisionDataset):
                             x = word["quad"]["x1"], word["quad"]["x2"], word["quad"]["x3"], word["quad"]["x4"]
                             y = word["quad"]["y1"], word["quad"]["y2"], word["quad"]["y3"], word["quad"]["y4"]
                             if use_polygons:
+                                # (x, y) coordinates of top left, top right, bottom right, bottom left corners
                                 box = np.array([
                                     [x[0], y[0]],
                                     [x[1], y[1]],
                                     [x[2], y[2]],
                                     [x[3], y[3]],
-                                ], dtype=np.float32)
+                                ], dtype=np_dtype)
                             else:
-                                # Reduce 8 coords to 4
+                                # Reduce 8 coords to 4 -> xmin, ymin, xmax, ymax
                                 box = [min(x), min(y), max(x), max(y)]
                             _targets.append((word['text'], box))
 
