@@ -1,4 +1,4 @@
-# Copyright (C) 2021, Mindee.
+# Copyright (C) 2021-2022, Mindee.
 
 # This program is licensed under the Apache License version 2.
 # See LICENSE or go to <https://www.apache.org/licenses/LICENSE-2.0.txt> for full license details.
@@ -18,10 +18,12 @@ class SVT(VisionDataset):
     """SVT dataset from `"The Street View Text Dataset - UCSD Computer Vision"
     <http://vision.ucsd.edu/~kai/svt/>`_.
 
-    Example::
-        >>> from doctr.datasets import SVT
-        >>> train_set = SVT(train=True, download=True)
-        >>> img, target = train_set[0]
+    .. image:: https://github.com/mindee/doctr/releases/download/v0.5.0/svt-grid.png
+        :align: center
+
+    >>> from doctr.datasets import SVT
+    >>> train_set = SVT(train=True, download=True)
+    >>> img, target = train_set[0]
 
     Args:
         train: whether the subset should be the training one
@@ -58,6 +60,7 @@ class SVT(VisionDataset):
                 raise FileNotFoundError(f"unable to locate {os.path.join(tmp_root, name.text)}")
 
             if use_polygons:
+                # (x, y) coordinates of top left, top right, bottom right, bottom left corners
                 _boxes = [
                     [
                         [float(rect.attrib['x']), float(rect.attrib['y'])],
