@@ -1,21 +1,24 @@
-# Copyright (C) 2021, Mindee.
+# Copyright (C) 2021-2022, Mindee.
 
 # This program is licensed under the Apache License version 2.
 # See LICENSE or go to <https://www.apache.org/licenses/LICENSE-2.0.txt> for full license details.
 
-import matplotlib.pyplot as plt
-from matplotlib.figure import Figure
-import matplotlib.patches as patches
-import mplcursors
-from PIL import ImageFont, ImageDraw, Image
 from copy import deepcopy
-import numpy as np
+from typing import Any, Dict, List, Optional, Tuple, Union
+
 import cv2
-from typing import Tuple, List, Dict, Any, Union, Optional
+import matplotlib.patches as patches
+import matplotlib.pyplot as plt
+import mplcursors
+import numpy as np
+from matplotlib.figure import Figure
+from PIL import Image, ImageDraw
+from unidecode import unidecode
 
-from .common_types import BoundingBox, RotatedBbox
+from .common_types import BoundingBox, Polygon4P
+from .fonts import get_font
 
-__all__ = ['visualize_page', 'synthetize_page', 'draw_boxes']
+__all__ = ['visualize_page', 'synthesize_page', 'draw_boxes']
 
 
 def rect_patch(
@@ -26,17 +29,19 @@ def rect_patch(
     alpha: float = 0.3,
     linewidth: int = 2,
     fill: bool = True,
+    preserve_aspect_ratio: bool = False
 ) -> patches.Rectangle:
     """Create a matplotlib rectangular patch for the element
 
     Args:
         geometry: bounding box of the element
-        page_dimensions: dimensions of the Page
+        page_dimensions: dimensions of the Page in format (height, width)
         label: label to display when hovered
         color: color to draw box
         alpha: opacity parameter to fill the boxes, 0 = transparent
         linewidth: line width
         fill: whether the patch should be filled
+        preserve_aspect_ratio: pass True if you passed True to the predictor
 
     Returns:
         a rectangular Patch
@@ -49,6 +54,8 @@ def rect_patch(
     height, width = page_dimensions
     (xmin, ymin), (xmax, ymax) = geometry
     # Switch to absolute coords
+    if preserve_aspect_ratio:
+        width = height = max(height, width)
     xmin, w = xmin * width, (xmax - xmin) * width
     ymin, h = ymin * height, (ymax - ymin) * height
 
@@ -65,42 +72,41 @@ def rect_patch(
 
 
 def polygon_patch(
-    geometry: RotatedBbox,
+    geometry: np.ndarray,
     page_dimensions: Tuple[int, int],
     label: Optional[str] = None,
     color: Tuple[float, float, float] = (0, 0, 0),
     alpha: float = 0.3,
     linewidth: int = 2,
     fill: bool = True,
+    preserve_aspect_ratio: bool = False
 ) -> patches.Polygon:
     """Create a matplotlib polygon patch for the element
 
     Args:
         geometry: bounding box of the element
-        page_dimensions: dimensions of the Page
+        page_dimensions: dimensions of the Page in format (height, width)
         label: label to display when hovered
         color: color to draw box
         alpha: opacity parameter to fill the boxes, 0 = transparent
         linewidth: line width
         fill: whether the patch should be filled
+        preserve_aspect_ratio: pass True if you passed True to the predictor
 
     Returns:
         a polygon Patch
     """
 
-    if len(geometry) != 5 or any(not isinstance(elt, float) for elt in geometry):
+    if not geometry.shape == (4, 2):
         raise ValueError("invalid geometry format")
 
     # Unpack
     height, width = page_dimensions
-    x, y, w, h, a = geometry
-    # Switch to absolute coords
-    x, w = x * width, w * width
-    y, h = y * height, h * height
-    points = cv2.boxPoints(((x, y), (w, h), a))
+    geometry[:, 0] = geometry[:, 0] * (max(width, height) if preserve_aspect_ratio else width)
+    geometry[:, 1] = geometry[:, 1] * (max(width, height) if preserve_aspect_ratio else height)
 
     return patches.Polygon(
-        points,
+        geometry,
         fill=fill,
         linewidth=linewidth,
         edgecolor=(*color, alpha),
@@ -110,7 +116,7 @@ def polygon_patch(
 
 
 def create_obj_patch(
-    geometry: Union[BoundingBox, RotatedBbox],
+    geometry: Union[BoundingBox, Polygon4P, np.ndarray],
     page_dimensions: Tuple[int, int],
     **kwargs: Any,
 ) -> patches.Patch:
@@ -118,17 +124,18 @@ def create_obj_patch(
 
     Args:
         geometry: bounding box (straight or rotated) of the element
-        page_dimensions: dimensions of the page
+        page_dimensions: dimensions of the page in format (height, width)
 
     Returns:
         a matplotlib Patch
     """
     if isinstance(geometry, tuple):
-        if len(geometry) == 2:
+        if len(geometry) == 2:  # straight word BB (2 pts)
             return rect_patch(geometry, page_dimensions, **kwargs)  # type: ignore[arg-type]
-        elif len(geometry) == 5:
-            return polygon_patch(geometry, page_dimensions, **kwargs)  # type: ignore[arg-type]
-
+        elif len(geometry) == 4:  # rotated word BB (4 pts)
+            return polygon_patch(np.asarray(geometry), page_dimensions, **kwargs)  # type: ignore[arg-type]
+    elif isinstance(geometry, np.ndarray) and geometry.shape == (4, 2):  # rotated line
+        return polygon_patch(geometry, page_dimensions, **kwargs)  # type: ignore[arg-type]
     raise ValueError("invalid geometry format")
 
 
@@ -144,16 +151,15 @@ def visualize_page(
 ) -> Figure:
     """Visualize a full page with predicted blocks, lines and words
 
-    Example::
-        >>> import numpy as np
-        >>> import matplotlib.pyplot as plt
-        >>> from doctr.utils.visualization import visualize_page
-        >>> from doctr.models import ocr_db_crnn
-        >>> model = ocr_db_crnn(pretrained=True)
-        >>> input_page = (255 * np.random.rand(600, 800, 3)).astype(np.uint8)
-        >>> out = model([[input_page]])
-        >>> visualize_page(out[0].pages[0].export(), input_page)
-        >>> plt.show()
+    >>> import numpy as np
+    >>> import matplotlib.pyplot as plt
+    >>> from doctr.utils.visualization import visualize_page
+    >>> from doctr.models import ocr_db_crnn
+    >>> model = ocr_db_crnn(pretrained=True)
+    >>> input_page = (255 * np.random.rand(600, 800, 3)).astype(np.uint8)
+    >>> out = model([[input_page]])
+    >>> visualize_page(out[0].pages[0].export(), input_page)
+    >>> plt.show()
 
     Args:
         page: the exported Page of a Document
@@ -212,13 +218,16 @@ def visualize_page(
                             int(page['dimensions'][1] * word['geometry'][0][0]),
                             int(page['dimensions'][0] * word['geometry'][0][1])
                         )
-                    ax.text(
-                        *text_loc,
-                        word['value'],
-                        size=10,
-                        alpha=0.5,
-                        color=(0, 0, 1),
-                    )
+
+                    if len(word['geometry']) == 2:
+                        # We draw only if boxes are in straight format
+                        ax.text(
+                            *text_loc,
+                            word['value'],
+                            size=10,
+                            alpha=0.5,
+                            color=(0, 0, 1),
+                        )
 
         if display_artefacts:
             for artefact in block['artefacts']:
@@ -242,10 +251,10 @@ def visualize_page(
     return fig
 
 
-def synthetize_page(
+def synthesize_page(
     page: Dict[str, Any],
     draw_proba: bool = False,
-    font_size: int = 13,
+    font_family: Optional[str] = None,
 ) -> np.ndarray:
     """Draw a the content of the element page (OCR response) on a blank page.
 
@@ -253,10 +262,12 @@ def synthetize_page(
         page: exported Page object to represent
         draw_proba: if True, draw words in colors to represent confidence. Blue: p=1, red: p=0
         font_size: size of the font, default font = 13
+        font_family: family of the font
 
     Return:
-        A np array (drawn page)
+        the synthesized page
     """
+
     # Draw template
     h, w = page["dimensions"]
     response = 255 * np.ones((h, w, 3), dtype=np.int32)
@@ -267,20 +278,19 @@ def synthetize_page(
             for word in line["words"]:
                 # Get aboslute word geometry
                 (xmin, ymin), (xmax, ymax) = word["geometry"]
-                xmin, xmax = int(w * xmin), int(w * xmax)
-                ymin, ymax = int(h * ymin), int(h * ymax)
+                xmin, xmax = int(round(w * xmin)), int(round(w * xmax))
+                ymin, ymax = int(round(h * ymin)), int(round(h * ymax))
 
                 # White drawing context adapted to font size, 0.75 factor to convert pts --> pix
-                h_box, w_box = ymax - ymin, xmax - xmin
-                h_font, w_font = font_size, int(font_size * w_box / (h_box * 0.75))
-                img = Image.new('RGB', (w_font, h_font), color=(255, 255, 255))
+                font = get_font(font_family, int(0.75 * (ymax - ymin)))
+                img = Image.new('RGB', (xmax - xmin, ymax - ymin), color=(255, 255, 255))
                 d = ImageDraw.Draw(img)
-
                 # Draw in black the value of the word
-                d.text((0, 0), word["value"], font=ImageFont.load_default(), fill=(0, 0, 0))
-
-                # Resize back to box size
-                img = img.resize((w_box, h_box), Image.NEAREST)
+                try:
+                    d.text((0, 0), word["value"], font=font, fill=(0, 0, 0))
+                except UnicodeEncodeError:
+                    # When character cannot be encoded, use its unidecode version
+                    d.text((0, 0), unidecode(word["value"]), font=font, fill=(0, 0, 0))
 
                 # Colorize if draw_proba
                 if draw_proba:
@@ -300,7 +310,7 @@ def synthetize_page(
 def draw_boxes(
     boxes: np.ndarray,
     image: np.ndarray,
-    color: Optional[Tuple] = None,
+    color: Optional[Tuple[int, int, int]] = None,
     **kwargs
 ) -> None:
     """Draw an array of relative straight boxes on an image
@@ -308,6 +318,7 @@ def draw_boxes(
     Args:
         boxes: array of relative boxes, of shape (*, 4)
         image: np array, float32 or uint8
+        color: color to use for bounding box edges
     """
     h, w = image.shape[:2]
     # Convert boxes to absolute coords
